% EVALUATECELL  returns function values inside a cell for given coefficient vector
%
% function U = evaluateCell(i, j, nx, ny, p, u, xi, eta)
%
% i   index of the cell in x-direction
% j   index of the cell in y-direction
% nx  number of cells in x-direction
% ny  number of cells in y-direction
% p   polynomial degree
% u   solution vector (1 x ndof-matrix)
% xi  1 x N1-matrix of local coordinates in [-1,1] in x-direction
% eta 1 x N2-matrix of local coordinates in [-1,1] in y-direction
%
% U   N2 x N1-matrix with function values
%
function U = evaluateCell(i, j, nx, ny, p, u, xi, eta)

U = zeros(length(eta), length(xi));

% 1D shapefunctions
Nx = shapefct1D(p, xi);
Ny = shapefct1D(p, eta);

Nnodes = (nx-1)*(ny-1);   % number of nodes
Nedhor = nx*(ny-1)*(p-1); % number of dof on horizontal edges
Nedver = (nx-1)*ny*(p-1); % number of dof on vertical edges

% four nodes
J = (j-2)*(nx-1) + i - 1 + [0 1 nx-1 nx];
I = find([0 1 0 1]+i >= 2  & [0 0 1 1]+j >= 2 & ...
	 [0 1 0 1]+i <= nx & [0 0 1 1]+j <= ny); % Dirichlet bc

if ismember(1,I), U = U + Ny(1,:)'*Nx(1,:)*u(J(1));end  % left lower
if ismember(2,I), U = U + Ny(1,:)'*Nx(2,:)*u(J(2));end  % right lower
if ismember(3,I), U = U + Ny(2,:)'*Nx(1,:)*u(J(3));end  % left upper
if ismember(4,I), U = U + Ny(2,:)'*Nx(2,:)*u(J(4));end  % right upper

% horizontal edges
J = ((j-2)*nx + i - 1 + [0 nx])*(p-1) + 1;
I = find([0 1]+j >= 2 & [0 1]+j <= ny);  % Dirichlet bc
J = J + Nnodes;

if ismember(1,I), U = U + Ny(1,:)'*(Nx(3:p+1,:)'*u(J(1)+[0:p-2])')';end % lower
if ismember(2,I), U = U + Ny(2,:)'*(Nx(3:p+1,:)'*u(J(2)+[0:p-2])')';end % upper

% vertical edges
J = ((j-1)*(nx-1) + i - 2 + [0 1])*(p-1) + 1;
I = find([0 1]+i >= 2 & [0 1]+i <= nx);  % Dirichlet bc
J = J + Nnodes + Nedhor;

if ismember(1,I), U = U + (u(J(1)+[0:p-2])*Ny(3:p+1,:))'*Nx(1,:); end % left
if ismember(2,I), U = U + (u(J(2)+[0:p-2])*Ny(3:p+1,:))'*Nx(2,:); end % right

% interior
J = ((j-1)*nx + i-1)*(p-1)^2 + Nnodes + Nedhor + Nedver + 1;
U = U + Ny(3:p+1,:)'*(Nx(3:p+1,:)'*reshape(u(J+[0:(p-1)^2-1]), p-1, p-1))';

return;